import QtQuick 1.1
import com.nokia.meego 1.0
import "script/settings.js" as AppSettings

Item {
    id: newsListContainer

    property int lastLoadedPage: 0

    property int loadedCategory: -1

    property bool loadedFavs: false

    property string loadedKeywords: ""
    
    property bool isWindowsteca: false
    
    property int prevNewsCount: 0
    property bool newsListEnded: false

    signal itemClicked(int id, int comments)

    Label {
        id: noResultsInfo
        width: parent.width - 20
        anchors.centerIn: parent
        opacity: 0
        
        font.pixelSize: settings.fontSizeL + settings.fontCorrection
        wrapMode: Text.WordWrap
        text: "Nessun articolo trovato."
        z: 100
    }

    UpdatableXmlListModel {
        id: newsModel
        query: "/root/item"

        XmlRole {
            name: "id"
            query: "id/number()"
            isKey: true
        }

        XmlRole {
            name: "title"
            query: "title/string()"
        }

        XmlRole {
            name: "thumb"
            query: "thumb/string()"
        }

        XmlRole {
            name: "pubDate"
            query: "pubDate/string()"
        }

        XmlRole {
            name: "author"
            query: "author/string()"
        }

        XmlRole {
            name: "commentsCount"
            query: "comments/number()"
        }

        XmlRole {
            name: "category"
            query: "tags/string()"
        }

        onStatusChanged: {
            if(source === "")
                return;

            if(status == XmlListModel.Loading) {
                if(busyIndicator)
                    busyIndicator.opacity = 1;
                return;
            }

            if(status == XmlListModel.Ready) {
                if(busyIndicator)
                    busyIndicator.opacity = 0;

                newsListView.opacity = 1;

                // If there are no search results, tell it to the user.
                if(newsListView.count === 0) {
                    noResultsInfo.opacity = 1;
                }
                else {
                    noResultsInfo.opacity = 0;
                    if(lastLoadedPage > 1) {
                        newsListView.positionViewAtIndex(
                                    newsListView.count - nokioteca.getPaginationLength(),
                                    ListView.End);
                    }
                }

                window.canCloseSplaschScreen = true;

                newsListEnded = (newsListView.count === prevNewsCount);
                prevNewsCount = newsListView.count;
                
                newsListView.interactive = true;
                return;
            }

            if(status == XmlListModel.Error) {
                if(busyIndicator)
                    busyIndicator.opacity = 0;
                noResultsInfo.text = "Si è verificato un errore.\n" +
                        "Verifica di essere connesso ad internet e che i server di " +
                        (isWindowsteca ? "Windowsteca.net" : "Nokioteca.net ") +
                        " siano funzionanti.";
                noResultsInfo.opacity = 1;
                noResultsInfo.color = settings.errorColor
                newsListView.opacity = 0;
                window.canCloseSplaschScreen = true;
            }
        }
    }

    Component {
        id: newsItem

        Item {
            property bool isFavourite: AppSettings.isFavourite(id, isWindowsteca)

            id: root
            width: parent.width
            height: childrenRect.height

            Item {
                width: parent.width

                height: itemDesc.height + 10 + settings.fontCorrection

                BorderImage {
                    id: background
                    anchors.fill: parent
                    // Fill page borders
                    anchors.leftMargin: -newsListContainer.anchors.leftMargin
                    anchors.rightMargin: -newsListContainer.anchors.rightMargin
                    visible: listItemMouseArea.pressed
                    source: "image://theme/meegotouch-list" + (theme.inverted ? "-inverted" : "") + "-background-pressed-center"
                }

                Rectangle {
                    height: 1
                    color: theme.inverted ? "#444444" : "#A9A9A9"
                    anchors {
                        bottom: parent.bottom
                        left: parent.left
                        right: parent.right
                    }
                }

                Item {
                    anchors {
                        right: parent.right
                        top: parent.top
                        rightMargin: settings.smallMargin
                        topMargin: settings.smallMargin
                    }
                    width: 25
                    height: 25
                    visible: commentsCount > 0

                    Rectangle {
                        anchors.fill: parent
                        color: (isFavourite && AppSettings.getPrevFavComments(id, isWindowsteca) < commentsCount) ? "red" : settings.nokiotecaBlue
                        radius: 10
                    }

                    Label {
                        anchors.centerIn: parent
                        text: commentsCount
                        font.pixelSize: commentsCount > 99 ? settings.fontSizeXS : settings.fontSizeS
                        font.weight: Font.DemiBold
                        color: settings.lightColor
                    }
                }

                Column {
                    id: itemDesc

                    anchors {
                        left: parent.left
                        right: parent.right
                        leftMargin: settings.smallMargin
                        rightMargin: settings.mediumMargin
                        verticalCenter: parent.verticalCenter
                    }

                    spacing: 5

                    Item {
                        id: itemMainInfos

                        width: parent.width
                        height: Math.max(thumbnailImage.height, newsTitle.height)

                        Image {
                            id: thumbnailImage
                            anchors {
                                left: parent.left
                                verticalCenter: parent.verticalCenter
                            }
                            width: settings.newsPreviewPicturesSize
                            sourceSize.width: width
                            fillMode: Image.PreserveAspectFit

                            source: thumb !== "" ? thumb : "qrc:images/icon.svg"
                        }

                        Text {
                            id: newsTitle
                            anchors {
                                left: thumbnailImage.right
                                leftMargin: settings.smallMargin
                                right: parent.right
                                rightMargin: 2 * settings.mediumMargin
                            }
                            wrapMode: Text.WordWrap
                            color: window.platformWhite ? settings.darkColor : settings.lightColor
                            verticalAlignment: Text.AlignVCenter
                            font.pixelSize: settings.fontSizeM + settings.fontCorrection

                            text: title
                        }
                    }
                    
                    Item {
                        id: itemSecInfos

                        width: parent.width
                        anchors {
                            left: parent.left
                            right: parent.right
                        }

                        height: 20

                        Text {
                            id: newsAuthor

                            anchors {
                                top: parent.top
                                bottom: parent.bottom
                                left: parent.left
                                right: newsDate.left
                                rightMargin: settings.smallMargin
                            }

                            elide: Text.ElideRight
                            color: window.platformWhite ? settings.nokiotecaBlue : settings.nokiotecaYellow
                            verticalAlignment: Text.AlignVCenter
                            font.pixelSize: settings.fontSizeXS + settings.fontCorrection
                            
                            text: author + " | " + category
                        }

                        Text {
                            id: newsDate

                            anchors {
                                top: parent.top
                                bottom: parent.bottom
                                right: parent.right
                            }

                            horizontalAlignment: Text.AlignRight
                            verticalAlignment: Text.AlignVCenter
                            
                            color: window.platformWhite ? settings.darkColor : settings.lightColor
                            
                            font.pixelSize: settings.fontSizeXS + settings.fontCorrection
                            font.italic: true

                            text: nokioteca.getPrettyDate(pubDate)
                        }
                    }
                }

                Image {
                     source: "image://theme/icon-m-common-drilldown-arrow" + (theme.inverted ? "-inverse" : "")
                     anchors.right: parent.right
                     anchors.verticalCenter: parent.verticalCenter
                 }

                MouseArea {
                    id: listItemMouseArea
                    anchors.fill: background

                    onPressAndHold:
                        clicked(mouse)

                    onClicked: {
                        // Item selected
                        if(busyIndicator)
                            busyIndicator.opacity = 1;

                        newsListView.currentIndex = index;
                        newsListContainer.itemClicked(id, commentsCount);
                    }
                }
            }

            ListView.onRemove: SequentialAnimation {
                PropertyAction { target: root; property: "ListView.delayRemove"; value: true }
                NumberAnimation {
                    target: root
                    property: "opacity"
                    from: 1
                    to: 0
                    duration: 200
                    easing.type: Easing.Linear
                }
                PropertyAction { target: root; property: "ListView.delayRemove"; value: false }
            }

            ListView.onAdd: NumberAnimation {
                target: root
                property: "opacity"
                from: 0
                to: 1
                duration: 250
                easing.type: Easing.Linear
            }
        }
    }

    ListView {
        id: newsListView

        anchors.fill: parent
        model: newsModel
        delegate: newsItem
        snapMode: ListView.SnapToItem
        clip: true
        cacheBuffer: 2000

        onMovementEnded: {
            if(loadedFavs || newsListEnded)
                return;
                
            if(atYEnd && count >= nokioteca.getPaginationLength()) {
                newsListView.interactive = false;
                if(loadedCategory === -1)
                    if(loadedKeywords.length > 0)
                        getNewsByKeywords(loadedKeywords)
                    else
                        getNews();
                else
                    getNewsByCategory(loadedCategory);
            }
        }
    }

    ScrollDecorator {
        flickableItem: newsListView
    }

    function reload() {
        lastLoadedPage = 0;
        newsListView.opacity = 1;
        var url;
        if(loadedCategory === -1) {
            if(loadedFavs) {
                url = (isWindowsteca ?
                           nokioteca.getWindowstecaFavouritesUrl(AppSettings.getFavouritesIds(isWindowsteca)) :
                           nokioteca.getFavouritesUrl(AppSettings.getFavouritesIds(isWindowsteca))
                       );                   
            }
            else {
                if(loadedKeywords.length > 0) {
                    url = (isWindowsteca ?
                               nokioteca.getWindowstecaSearchUrl(lastLoadedPage, keywords) :
                               nokioteca.getSearchUrl(lastLoadedPage, keywords)
                           );
                }
                else {
                    url = (isWindowsteca ?
                               nokioteca.getWindowstecaNewsListUrl(lastLoadedPage) :
                               nokioteca.getNewsListUrl(lastLoadedPage)
                           );
                }
            }

        }
        else {
            url = nokioteca.getNewsListUrl(lastLoadedPage, catId);
        }

        lastLoadedPage++;
        newsModel.reload(url);
    }

    /* Get the latest news */
    function getNews() {
        var url = (isWindowsteca ?
                       nokioteca.getWindowstecaNewsListUrl(lastLoadedPage) :
                       nokioteca.getNewsListUrl(lastLoadedPage)
                   );

        lastLoadedPage++;
        newsModel.source = url;
    }

    /* Get the latest news by category */
    function getNewsByCategory(catId) {
        var url = nokioteca.getNewsListUrl(lastLoadedPage, catId);

        lastLoadedPage++;
        loadedCategory = catId;
        newsModel.source = url;
    }

    /* Get the favourites news */
    function getFavouritesNews(favsList) {
        var url = (isWindowsteca ?
                       nokioteca.getWindowstecaFavouritesUrl(favsList) :
                       nokioteca.getFavouritesUrl(favsList)
                    );

        lastLoadedPage++;
        loadedFavs = true;
        newsModel.source = url;
    }

    /* Get news by keyword */
    function getNewsByKeywords(keywords) {
        var url = (isWindowsteca ? 
                   nokioteca.getWindowstecaSearchUrl(lastLoadedPage, keywords) :
                   nokioteca.getSearchUrl(lastLoadedPage, keywords)
                );

        lastLoadedPage++;
        loadedKeywords = keywords;
        newsModel.source = url;
    }
}
