import QtQuick 1.1
import com.nokia.extras 1.1
import com.nokia.meego 1.0

import "script/settings.js" as AppSettings
import "script/common.js" as CommonScript
import "script/ajax.js" as AjaxScript

PageStackWindow {
    id: window

    property bool platformWhite: false

    property bool isShowingSplashScreen: true
    property bool canCloseSplaschScreen: false
    property variant busySplash
    
    showStatusBar: !isShowingSplashScreen
    showToolBar: !isShowingSplashScreen
    initialPage: Qt.resolvedUrl("NewsPage.qml");

    Component {
        id: busySplashComp

        LoadingAppIndicator {
            id: busy

            function __hideSplash() {
                busy.opacity = 0;
                window.isShowingSplashScreen = false;
            }

            width: 480
            height: 854
            rotation: -90
            anchors.centerIn: parent

            // Get rid of the fake splash for good, when loading is done!
            onDismissed: busySplash.destroy();

            Component.onCompleted: {
                if(window.canCloseSplaschScreen)
                    __hideSplash();
            }

            Connections {
                target: window
                onCanCloseSplaschScreenChanged: {
                    if(canCloseSplaschScreen)
                        __hideSplash();
                }
            }
        }
    }

    Settings { id: settings }
    
    // General busy indicator which can be actived by the pages in the page
    // stack
    BusyIndicator {
        id: busyIndicator
        anchors.centerIn: parent
        opacity: 0
        running: opacity > 0
        z: 100

        platformStyle: BusyIndicatorStyle { size: "large" }

        Behavior on opacity {
            NumberAnimation {
                duration: 300;
            }
        }
    }

    InfoBanner {
        id: infoBanner
    }

    Loader {
        id: zoomedImageLoader

        anchors.fill: parent

        function show(source) {
            if(!item)
                sourceComponent = zoomedImageComponent;

            item.source = source;
        }

        function showGallery(galleryItem) {
            if(!item)
                sourceComponent = zoomedImageComponent;

            item.source = galleryItem.model.get(galleryItem.currentIndex).picUrl;
            item.galleryItem = galleryItem;
            galleryItem.interactive = false;
        }
    }

    Component {
        id: zoomedImageComponent

        Item {
            id: zoomedImage

            property alias source: theZoomedImage.source
            property Item galleryItem: null

            opacity: 0

            state: "INVISIBLE"

            Rectangle {
                anchors.fill: parent
                opacity: 0.5
                color: window.platformWhite ? settings.darkColor : settings.lightColor
            }

            MouseArea {
                id: previousPic
                z: 10
                
                anchors {
                    left: parent.left
                    top: parent.top
                    bottom: parent.bottom
                }
                width: 40
                
                visible: zoomedImage.galleryItem !== null
                enabled: theZoomedImage.status == Image.Ready || theZoomedImage.status == Image.Error

                onClicked: {
                    busyIndicator.opacity = 1;
                    zoomedImage.galleryItem.decrementCurrentIndex();
                    changePictureAnimation.startAnimation(zoomedImage.galleryItem.model.get(zoomedImage.galleryItem.currentIndex).picUrl);
                }

                Image {
                    anchors.centerIn: parent
                    source: "image://theme/icon-m-toolbar-previous-white";
                }
            }

            MouseArea {
                id: nextPic
                z: 10
                
                anchors {
                    right: parent.right
                    top: parent.top
                    bottom: parent.bottom
                }
                width: 40
                
                visible: zoomedImage.galleryItem !== null
                enabled: theZoomedImage.status == Image.Ready || theZoomedImage.status == Image.Error

                onClicked: {
                    busyIndicator.opacity = 1;
                    zoomedImage.galleryItem.incrementCurrentIndex();
                    changePictureAnimation.startAnimation(zoomedImage.galleryItem.model.get(zoomedImage.galleryItem.currentIndex).picUrl);
                }

                Image {
                    anchors.centerIn: parent
                    source: "image://theme/icon-m-toolbar-next-white";
                }
            }

            ZoomableImage {
                id: theZoomedImage
                anchors.fill: parent
                
                onStatusChanged: {
                    if(status === Image.Error) {
                        zoomedImage.state = "ERROR"
                        busyIndicator.opacity = 0;
                    }
                    else if(status === Image.Ready) {
                        zoomedImage.state = "VISIBLE"
                        busyIndicator.opacity = 0;
                    }
                }

                onClicked: zoomedImage.state = "INVISIBLE"
            }

            onSourceChanged: theZoomedImage.source = source

            SequentialAnimation {
                id: changePictureAnimation
                property string futureSource

                function startAnimation(theSource) {
                    futureSource = theSource;
                    start();
                }

                NumberAnimation {
                    target: theZoomedImage
                    property: "scale"
                    to: 0
                    running: false
                    duration: 150
                }
                ScriptAction { script: theZoomedImage.scale = 1 }
                ScriptAction { script: zoomedImage.source = "" }
                ScriptAction { script: zoomedImage.source = changePictureAnimation.futureSource }
            }

            Rectangle {
                id: errorLoading
                anchors.centerIn: parent
                color: "black"
                visible: false
                width: 300
                height: 80

                MouseArea {
                    anchors.fill: parent
                    onClicked: zoomedImage.state = "INVISIBLE"
                }

                Row {
                    anchors.centerIn: parent
                    spacing: settings.mediumMargin
                    Image {
                        id: imageErrorLogo
                        anchors.verticalCenter: parent.verticalCenter
                        source: "qrc:images/ko.png"
                    }

                    Label {
                        anchors.verticalCenter: parent.verticalCenter
                        color: settings.errorColor
                        text: "Errore nel caricamento\ndell'immagine"
                    }
                }
            }

            Rectangle {
                anchors {
                    bottom: parent.bottom
                    left: parent.left
                    right: parent.right
                }
                height: childrenRect.height
                color: window.platformWhite ? settings.darkColor : settings.lightColor

                Label {
                    anchors.horizontalCenter: parent.horizontalCenter
                    text: "Clicca sullo schermo per chiudere"
                    font.pixelSize: settings.fontSizeXS + (settings.fontCorrection / 2)
                    color: window.platformWhite ? settings.lightColor : settings.darkColor
                }
            }

            states: [
                State {
                     name: "VISIBLE"
                     PropertyChanges { target: zoomedImage; opacity: 1 }
                },
                State {
                     name: "INVISIBLE"
                     PropertyChanges { target: zoomedImage; opacity: 0 }
                     PropertyChanges { target: errorLoading; visible: false }
                     PropertyChanges { target: zoomedImageLoader; sourceComponent: "" }
                },
                State {
                    name: "ERROR"
                    PropertyChanges { target: zoomedImage; opacity: 1 }
                    PropertyChanges { target: errorLoading; visible: true }
                }
            ]
            transitions: [
                Transition {
                    from: "INVISIBLE"
                    to: "*"
                    NumberAnimation { properties: "opacity"; duration: 500 }
                },
                Transition {
                    from: "*"
                    to: "INVISIBLE"
                    SequentialAnimation {
                        NumberAnimation { properties: "opacity"; duration: 300 }
                        ScriptAction { script: {
                                            if(zoomedImage.galleryItem)
                                                zoomedImage.galleryItem.interactive = true;
                                            zoomedImage.galleryItem = null;
                                            zoomedImage.source = "";
                            }
                        }
                    }
                }
            ]
        }
    }

    Menu {
        id: smallMenu

        visualParent: pageStack

        MenuLayout {
            MenuItem {
                text: "Impostazioni"
                onClicked: {
                    if(!pageStack.busy)
                        pageStack.push(Qt.resolvedUrl("SettingsUIPage.qml"));
                }
            }

            MenuItem {
                text: "Esci"
                onClicked: Qt.quit();
            }
        }
    }

    Menu {
        id: appMenu

        visualParent: pageStack

        MenuLayout {
            MenuItem {
                text: "Preferiti"
                onClicked: {
                    if(!pageStack.busy) {
                        var favsPage = pageStack.push(Qt.resolvedUrl("FavouritesPage.qml"),
                                       { isWindowsteca: typeof pageStack.currentPage.isWindowsteca !== "undefined" ?
                                                            pageStack.currentPage.isWindowsteca :
                                                            false });

                        favsPage.getFavouritesNews();
                    }
                }
            }
            
            MenuItem {
                text: "Impostazioni"
                onClicked: {
                    if(!pageStack.busy)
                        pageStack.push(Qt.resolvedUrl("SettingsUIPage.qml"));
                }
            }
            
            MenuItem {
                visible: (typeof pageStack.currentPage.searchEnabled !== "undefined" &&
                          pageStack.currentPage.searchEnabled)
                text: "Cerca una notizia"
                onClicked: {
                    searchDialogLoader.show();
                }
            }
            
            MenuItem {
                visible: settings.allowRecommendNews
                
                text: "Segnala una notizia"
                onClicked: {
                    if(!pageStack.busy)
                        Qt.openUrlExternally(AppSettings.getRecommendNewsUrl());
                }
            }
            
            MenuItem {
                text: "Seguici..."
                onClicked: {
                    if(!pageStack.busy)
                        pageStack.push(Qt.resolvedUrl("FollowUsPage.qml"));
                }
            }

            MenuItem {
                text: "Informazioni"
                onClicked: {
                    if(!pageStack.busy)
                        pageStack.push(Qt.resolvedUrl("AboutPage.qml"));
                }
            }

            MenuItem {
                text: "Esci"
                onClicked: Qt.quit();
            }
        }
    }

    // event preventer when page transition is active
    MouseArea {
        anchors.fill: parent
        enabled: pageStack.busy
    }

    Component.onCompleted: {
        var comp = busySplashComp;
        if(comp.status === Component.Ready) {
            busySplash = comp.createObject(window);
        }

        AppSettings.initialize();
        window.platformWhite = AppSettings.useWhiteTheme();
        if(theme.inverted !== !window.platformWhite)
            theme.inverted = !window.platformWhite;
        settings.fontCorrection = AppSettings.getFontSizeCorrection();
        settings.addToFavOnReply = AppSettings.addFavOnReply();
        settings.commentsUniversalEnabled = AppSettings.getCommentsUniversalEnabled();
        settings.searchUniversalEnabled = AppSettings.getSearchUniversalEnabled();
        settings.allowRecommendNews = (AppSettings.getRecommendNewsUrl().indexOf("http") >= 0);
        
        var now = (new Date()).getTime();
        if(now - AppSettings.getServerSettingCheckingTime() > 24*60*60*1000) {
            getServerSettings();
            AppSettings.setServerSettingCheckingTime(now);
        }
    }
    
    Loader {
        id: searchDialogLoader

        function show() {
            if(!item)
                sourceComponent = searchDialogComponent;
            item.open();
        }
    }

    Component {
        id: searchDialogComponent

        Item {
            CommonDialog {
                id: searchDialog

                titleText: "Ricerca articolo"
                buttonTexts: ["Cerca", "Annulla"]

                content: Item {
                    anchors {
                        left: parent.left
                        right: parent.right
                    }

                    height: childrenRect.height

                    TextField {
                        id: searchText

                        anchors {
                            left: parent.left
                            right: parent.right
                            top: parent.top
                            leftMargin: settings.mediumMargin
                            rightMargin: settings.mediumMargin
                        }

                        inputMethodHints: Qt.ImhNoPredictiveText
                        maximumLength: 40
                        placeholderText: "Inserisci la stringa da cercare"
                    }
                }

                onButtonClicked: {
                    var keywords = searchText.text;
                    searchText.text = "";
                    if(index === 0 && keywords.length > 0) {
                        if(!pageStack.busy) {
                            var searchResultsPage;
                            if(typeof pageStack.currentPage.searchedKeyword !== "undefined") {
                                searchResultsPage =
                                    pageStack.replace(Qt.resolvedUrl("SearchResultsPage.qml"),
                                                            { searchedKeyword: keywords,
                                                              isWindowsteca: pageStack.currentPage.isWindowsteca }
                                    );
                            }
                            else {
                                searchResultsPage =
                                    pageStack.push(Qt.resolvedUrl("SearchResultsPage.qml"),
                                                            { searchedKeyword: keywords,
                                                              isWindowsteca: pageStack.currentPage.isWindowsteca }
                                    );
                            }

                            searchResultsPage.searchResultsView.getNewsByKeywords(keywords);
                        }
                    }
                    else {
                        reject();
                    }
                }
            }
            
            function open() {
                searchText.text = "";
                searchDialog.open();
            }
        }
    }
    
    function getServerSettings() {
        var req = AjaxScript.createXMLHttpRequest();

        if(req) {
            var url = nokioteca.getSettingsUrl();

            AjaxScript.loadXMLDoc(req, url, parseServerSettings);
        }
    }

    function parseServerSettings(xml, text) {
        if(xml === null)
            return;

        var universalComments;
        var universalSearch;
        var reportNewsUrl;
        
        try {
            universalComments = (CommonScript.getElementsByTagName(
                xml.documentElement, "uce")[0].firstChild.nodeValue !== "false");
        } catch(err) {
            universalComments = true;
        }

        try {
            universalSearch = (CommonScript.getElementsByTagName(
                        xml.documentElement, "use")[0].firstChild.nodeValue !== "false");
        } catch(err) {
            universalSearch = true;
        }

        try {
            reportNewsUrl = CommonScript.getElementsByTagName(
                xml.documentElement, "rnu")[0].firstChild.nodeValue;
        } catch(err) {
            reportNewsUrl = "";
        }
        
        if(universalComments !== settings.commentsUniversalEnabled) {
            AppSettings.setCommentsUniversalEnabled(universalComments);
            settings.commentsUniversalEnabled = universalComments;
        }
        if(universalSearch !== settings.searchUniversalEnabled) {
            AppSettings.setSearchUniversalEnabled(universalSearch);
            settings.searchUniversalEnabled = universalSearch;
        }

        if(reportNewsUrl.length > 0) {
            AppSettings.setRecommendNewsUrl(reportNewsUrl);
            settings.allowRecommendNews = (reportNewsUrl.indexOf("http") >= 0);
        }
    }
}
