import QtQuick 1.1
import com.nokia.meego 1.0

Item {
    property int newsId // The ID of the news currently on the screen
    property int prevCommentsCount: 0

    anchors.fill: parent

    CommentXmlListModel {
        id: commentsModel

        query: "/root/item"

        onStatusChanged: {
            if(status == XmlListModel.Null) {
                commentsListView.opacity = 0;
                commentsLoadingErrors.opacity = 0;
                return;
            }

            if(status == XmlListModel.Loading) {
                if(busyIndicator)
                    busyIndicator.opacity = 1;
                commentsLoadingErrors.opacity = 0;
                commentsListView.opacity = 0;
                return;
            }

            if(status == XmlListModel.Error) {
                if(busyIndicator)
                    busyIndicator.opacity = 0;

                commentsLoadingErrors.text = "Si è verificato un errore nel recupero dei commenti.\n" +
                        "Verifica di essere connesso ad internet e che i server di " +
                        (isWindowsteca ? "Windowsteca.net" : "Nokioteca.net ") +
                        " siano funzionanti.";
                        
                commentsLoadingErrors.opacity = 1;
                commentsListView.opacity = 0;
                return;
            }

            if(status == XmlListModel.Ready) {
                if(busyIndicator)
                    busyIndicator.opacity = 0;

                commentsLoadingErrors.opacity = 0;

                if(count === 1 && get(0).commentId === -1)
                    newsDetailPage.refreshCommentsCount(prevCommentsCount, 0);
                else
                    newsDetailPage.refreshCommentsCount(prevCommentsCount, count);
                commentsListView.opacity = 1;
            }
        }
    }

    Component {
        id: singleCommentContainer

        Item {
            id: root
            anchors {
                left: parent.left
                right: parent.right
            }

            height: commentSkeleton.height + 10

            AComment {
                id: commentSkeleton

                anchors {
                    left: parent.left
                    right: parent.right
                    leftMargin: settings.smallMargin
                    rightMargin: settings.mediumMargin
                    verticalCenter: parent.verticalCenter
                }

                myId: commentId
                myText: commentText
                author: commentAuthor
                date: nokioteca.getPrettyDate(commentDate, true);
                level: commentLevel
                avatarSource: authorAvatar
                replyButtonEnabled: newsDetailPage.commentsEnabled
                isEmpty: (commentId === -1)

                Component.onCompleted:
                    commentSkeleton.answerRequested.connect(commentsListView.replyRequested)
            }
        }
    }

    Label {
        id: commentsLoadingErrors

        anchors {
            top: parent.top
            left: parent.left
            right: parent.right
            bottomMargin: settings.mediumMargin
        }

        z: 100
        
        font.pixelSize: settings.fontSizeL + settings.fontCorrection
        wrapMode: Text.WordWrap
        color: settings.errorColor

        opacity: 0
    }

    ListView {
        id: commentsListView

        signal replyRequested(int toPost, string toUser, string toText)

        anchors.fill: parent

        Behavior on y { PropertyAnimation { duration: 200 } }

        model: commentsModel
        delegate: singleCommentContainer
        snapMode: ListView.NoSnap
        clip: true

        cacheBuffer: 2500

        footer: NewCommentInputMask {
            id: newCommentInputMask

            replyToNewsId: newsId

            anchors {
                left: parent.left
                right: parent.right
                leftMargin: settings.smallMargin
                rightMargin: settings.mediumMargin
                topMargin: settings.mediumMargin
            }

            commentsEnabled: newsDetailPage.commentsEnabled

            Component.onCompleted: {
                commentsListView.replyRequested.connect(newCommentInputMask.setAsReply)
            }
        }

        onCountChanged: commentsListView.positionViewAtBeginning()

        onContentHeightChanged: {
            moveListButton.enabled = commentsListView.contentHeight > (commentsListView.height - tools.height);
        }

        onAtYEndChanged: moveListButton.rotation = moveListButton.enabled && atYEnd ? 180 : 0
        
        Behavior on opacity {
            NumberAnimation {
                duration: 250
                easing.type: Easing.Linear
            }
        }
    }

    ScrollDecorator {
        flickableItem: commentsListView
    }

    function scrollList(theButton) {
        if(commentsListView.atYEnd)
            commentsListView.positionViewAtBeginning();
        else
            commentsListView.positionViewAtEnd();

        theButton.rotation = commentsListView.count && commentsListView.atYEnd ? 180 : 0;
    }

    function sortByTime(byTime) {
        commentsModel.changeCommentsSorting(byTime);
    }

    function refreshComments() {
        if(commentsModel.count === 1 && commentsModel.get(0).commentId === -1)
            prevCommentsCount = 0;
        else
            prevCommentsCount = commentsModel.count;
            
        commentsModel.reload();
    }

    function getComments(news_id) {
        var url = (isWindowsteca ?
                       nokioteca.getWindowstecaCommentsUrl(news_id) :
                       nokioteca.getCommentsUrl(news_id)
                   );
                   
        newsId = news_id;

        if(commentsModel.source !== url)
            commentsModel.source = url;
    }
}
