import QtQuick 1.1
import com.nokia.meego 1.0
import "UIConstants.js" as UI

FocusScope {
    id: root

    // Common public API
    property alias font: textEdit.font
    property alias cursorPosition: textEdit.cursorPosition
    property alias horizontalAlignment: textEdit.horizontalAlignment
    property alias verticalAlignment: textEdit.verticalAlignment
    property alias readOnly: textEdit.readOnly
    property alias selectedText: textEdit.selectedText
    property alias selectionEnd: textEdit.selectionEnd
    property alias selectionStart: textEdit.selectionStart
    property alias text: textEdit.text
    property alias textFormat: textEdit.textFormat
    property alias wrapMode: textEdit.wrapMode

    function copy() {
        textEdit.copy()
    }

    function paste() {
        textEdit.paste()
    }

    function cut() {
        textEdit.cut()
    }

    function select(start, end) {
        textEdit.select(start, end)
    }

    function selectAll() {
        textEdit.selectAll()
    }

    function selectWord() {
        textEdit.selectWord()
    }

    function positionAt(x, y) {
        var p = mapToItem(textEdit, x, y);
        return textEdit.positionAt(p.x, p.y)
    }

    function positionToRectangle(pos) {
        var rect = textEdit.positionToRectangle(pos)
        var point = mapFromItem(textEdit, rect.x, rect.y)
        rect.x = point.x; rect.y = point.y
        return rect;
    }

    function closeSoftwareInputPanel() {
        platformCloseSoftwareInputPanel()
    }

    function platformCloseSoftwareInputPanel() {
        inputContext.simulateSipClose();
        textEdit.closeSoftwareInputPanel();
    }

    function openSoftwareInputPanel() {
        platformOpenSoftwareInputPanel()
    }

    function platformOpenSoftwareInputPanel() {
        inputContext.simulateSipOpen();
        textEdit.openSoftwareInputPanel();
    }
    property alias placeholderText: placeholder.text
    
    property real maxImplicitWidth: parent ? parent.width : screen.width
    property real maxImplicitHeight: parent ? parent.height : screen.height

    implicitWidth: {
        var preferredWidth = Math.max(flick.contentWidth, privy.minImplicitWidth)
        preferredWidth += container.anchors.leftMargin + container.anchors.rightMargin
        return Math.min(preferredWidth, root.maxImplicitWidth)
    }

    implicitHeight: {
        var preferredHeight = Math.max(flick.contentHeight, privy.minImplicitHeight)
        preferredHeight += container.anchors.topMargin + container.anchors.bottomMargin
        return Math.min(preferredHeight, root.maxImplicitHeight)
    }

    QtObject {
        id: privy
        // TODO: More consistent minimum width for empty TextArea than 20 * " " on current font?
        property real minImplicitWidth: placeholder.text ? placeholder.model.paintedWidth
                                                         : platformStyle.textWidth("                    ", style.current.get("font"))
        property real minImplicitHeight: placeholder.text ? placeholder.model.paintedHeight
                                                          : platformStyle.fontHeight(platformStyle.current.get("font"))
    }
    
    property QtObject platformStyle: TextAreaStyle {}

    
    onActiveFocusChanged: {
        if(activeFocus &&
            !readOnly) {
            platformOpenSoftwareInputPanel();
        } else if (!activeFocus) {
            if (!readOnly)
                platformCloseSoftwareInputPanel();
        }
    }
    
    BorderImage {
        id: background
            source: readOnly?
                platformStyle.backgroundDisabled:
            textEdit.activeFocus? 
                platformStyle.backgroundSelected:
                platformStyle.background

        anchors.fill: parent
        border.left: root.platformStyle.backgroundCornerMargin; border.top: root.platformStyle.backgroundCornerMargin
        border.right: root.platformStyle.backgroundCornerMargin; border.bottom: root.platformStyle.backgroundCornerMargin
    }

    Item {
        id: container
        anchors {
            leftMargin: UI.PADDING_XLARGE; rightMargin: UI.PADDING_XLARGE
            topMargin: UI.PADDING_XLARGE; bottomMargin: UI.PADDING_XLARGE
        }

        x: anchors.leftMargin
        y: anchors.topMargin
        height: parent.height - anchors.bottomMargin - y
        width: parent.width - anchors.rightMargin - x
        Text {
            id: placeholder
            objectName: "placeholder"
            
            property variant model: Text {
                font: root.platformStyle.textFont
                text: placeholder.text
                visible: false
                wrapMode: textEdit.wrapMode
                horizontalAlignment: textEdit.horizontalAlignment
                verticalAlignment: textEdit.verticalAlignment
                height: root.maxImplicitHeight - container.anchors.topMargin - container.anchors.bottomMargin
                width: root.maxImplicitWidth - container.anchors.leftMargin - container.anchors.rightMargin
                opacity: 0
            }

            color: root.platformStyle.promptTextColor
            font: root.platformStyle.textFont
            opacity: 0
            wrapMode: textEdit.wrapMode
            height: parent.height
            width: parent.width

            states: [
                State {
                    name: "unfocused"
                    // memory allocation optimization: cursorPosition is checked to minimize displayText evaluations
                    when: !root.activeFocus && textEdit.cursorPosition == 0 && !textEdit.text && placeholder.text && !textEdit.inputMethodComposing
                    PropertyChanges { target: placeholder; opacity: 1.0; }
                },
                State {
                    name: "focused"
                    // memory allocation optimization: cursorPosition is checked to minimize displayText evaluations
                    when: root.activeFocus && textEdit.cursorPosition == 0 && !textEdit.text && placeholder.text && !textEdit.inputMethodComposing
                    PropertyChanges { target: placeholder; opacity: 0.6; }
                }
            ]

            transitions: [
                Transition {
                    from: "unfocused"; to: "focused";
                    reversible: true
                    SequentialAnimation {
                        PauseAnimation { duration: 60 }
                        NumberAnimation { target: placeholder; properties: "opacity"; duration: 150 }
                    }
                },
                Transition {
                    from: "focused"; to: "";
                    reversible: true
                    SequentialAnimation {
                        PauseAnimation { duration:  60 }
                        NumberAnimation { target: placeholder; properties: "opacity"; duration: 100 }
                    }
                }
            ]
        }

        Flickable {
            id: flick

            function ensureVisible(rect) {
                if (Math.round(contentX) > Math.round(rect.x))
                    contentX = rect.x
                else if (Math.round(contentX + width) < Math.round(rect.x + rect.width))
                    contentX = rect.x + rect.width - width

                if (Math.round(contentY) > Math.round(rect.y))
                    contentY = rect.y
                else if (Math.round(contentY + height) < Math.round(rect.y + rect.height))
                     contentY = rect.y + rect.height - height
            }

            anchors.fill: parent
            boundsBehavior: Flickable.StopAtBounds
            clip: true
            contentHeight: textEdit.model.paintedHeight
            contentWidth: textEdit.model.paintedWidth

            // To avoid too early loading of ScrollDecorator when Flickable is not ready
            Component.onCompleted: { scrollDecorator.flickableItem = flick }

            TextEdit {
                id: textEdit
                objectName: "textEdit"

                property variant model: Text {
                    font: root.platformStyle.textFont
                    color: root.platformStyle.textColor
                    text: placeholder.text
                    horizontalAlignment: textEdit.horizontalAlignment
                    verticalAlignment: textEdit.verticalAlignment
                    wrapMode: textEdit.wrapMode
                    visible: false
                    opacity: 0
                    height: root.maxImplicitHeight - container.anchors.topMargin - container.anchors.bottomMargin
                    width: root.maxImplicitWidth - container.anchors.leftMargin - container.anchors.rightMargin
                }
                
                font: root.platformStyle.textFont
                color: root.platformStyle.textColor
                selectedTextColor: root.platformStyle.selectedTextColor
                selectionColor: root.platformStyle.selectionColor
                textFormat: TextEdit.AutoText
                wrapMode: TextEdit.Wrap
                // TODO: Use desktop text selection behaviour for now.
                selectByMouse: false
                height: container.height
                width: container.width
                // TODO: Make bug report?
                // Called too early (before TextEdit size is adjusted) delay ensureVisible call a bit
                onCursorRectangleChanged: {
                    delayedEnsureVisible.start()
                }
            }
        }

        ScrollDecorator {
            id: scrollDecorator
        }

        Timer {
            id: delayedEnsureVisible
            interval: 1
            onTriggered: flick.ensureVisible(textEdit.cursorRectangle)
        }
    }
}
