import QtQuick 1.1
import com.nokia.meego 1.0
import "UIConstants.js" as UI

FocusScope {
    id: root

    // Common public API
    property alias font: textEdit.font
    property alias cursorPosition: textEdit.cursorPosition
    property alias horizontalAlignment: textEdit.horizontalAlignment
    property alias verticalAlignment: textEdit.verticalAlignment
    property alias readOnly: textEdit.readOnly
    property alias selectedText: textEdit.selectedText
    property alias selectionEnd: textEdit.selectionEnd
    property alias selectionStart: textEdit.selectionStart
    property alias text: textEdit.text
    property alias textFormat: textEdit.textFormat
    property alias wrapMode: textEdit.wrapMode

    property QtObject platformStyle: TextAreaStyle {}
    
    function copy() {
        textEdit.copy()
    }

    function paste() {
        textEdit.paste()
    }

    function cut() {
        textEdit.cut()
    }

    function select(start, end) {
        textEdit.select(start, end)
    }

    function selectAll() {
        textEdit.selectAll()
    }

    function selectWord() {
        textEdit.selectWord()
    }

    function positionAt(x, y) {
        var p = mapToItem(textEdit, x, y);
        return textEdit.positionAt(p.x, p.y)
    }

    function positionToRectangle(pos) {
        var rect = textEdit.positionToRectangle(pos)
        var point = mapFromItem(textEdit, rect.x, rect.y)
        rect.x = point.x; rect.y = point.y
        return rect;
    }

    function openSoftwareInputPanel() {
        textEdit.openSoftwareInputPanel()
    }

    function closeSoftwareInputPanel() {
        textEdit.closeSoftwareInputPanel()
    }
    
    property alias placeholderText: placeholder.text
    
    property real platformMaxImplicitWidth: -1
    property real platformMaxImplicitHeight: -1
    property bool enabled: true // overriding due to QTBUG-15797 and related bugs

    implicitWidth: {
        var preferredWidth = placeholder.visible ? placeholder.model.paintedWidth
                                                 : flick.contentWidth
        preferredWidth = Math.max(preferredWidth, privy.minImplicitWidth)
        preferredWidth += container.horizontalMargins
        if (root.platformMaxImplicitWidth >= 0)
            return Math.min(preferredWidth, root.platformMaxImplicitWidth)
        return preferredWidth
    }

    implicitHeight: {
        var preferredHeight = placeholder.visible ? placeholder.model.paintedHeight
                                                  : flick.contentHeight
        preferredHeight += container.verticalMargins
        // layout spec gives minimum height (textFieldHeight) which includes required padding
        preferredHeight = Math.max(platformStyle.textFieldHeight, preferredHeight)
        if (root.platformMaxImplicitHeight >= 0)
            return Math.min(preferredHeight, root.platformMaxImplicitHeight)
        return preferredHeight
    }
    
    onWidthChanged: {
        // Detect when a width has been explicitly set. Needed to determine if the TextEdit should
        // grow horizontally or wrap. I.e. in determining the model's width. There's no way to get
        // notified of having an explicit width set. Therefore it's polled in widthChanged.
        privy.widthExplicit = true
    }
    
    Connections {
         target: screen
         onCurrentOrientationChanged: {
             delayedEnsureVisible.start()
             fade.start()
             scroll.start()
         }
    }

    QtObject {
        id: privy
        // TODO: More consistent minimum width for empty TextArea than 20 * " " on current font?
        property real minImplicitWidth: placeholder.text ? placeholder.model.paintedWidth
                                                         : platformStyle.textWidth("                    ", style.current.get("font"))
        property bool widthExplicit: false
        property bool wrap: privy.widthExplicit || root.platformMaxImplicitWidth >= 0
        property real wrapWidth: privy.widthExplicit ? root.width - container.horizontalMargins
                                                     : root.platformMaxImplicitWidth - container.horizontalMargins

        function bg_postfix() {
            if (root.readOnly || !root.enabled)
                return "uneditable"
            else if (textEdit.activeFocus)
                return "highlighted"
            else
                return "editable"
        }
    }
    
    BorderImage {
        id: background
        anchors.fill: parent
        source: readOnly?
                    platformStyle.backgroundDisabled:
                textEdit.activeFocus?
                    platformStyle.backgroundSelected:
                    platformStyle.background
        border {
            left: root.platformStyle.backgroundCornerMargin
            top: root.platformStyle.backgroundCornerMargin
            right: root.platformStyle.backgroundCornerMargin
            bottom: root.platformStyle.backgroundCornerMargin
        }
        smooth: true
    }
    
    /*property QtObject platformStyle: TextAreaStyle {}

    
    onActiveFocusChanged: {
        if(activeFocus &&
            !readOnly) {
            platformOpenSoftwareInputPanel();
        } else if (!activeFocus) {
            if (!readOnly)
                platformCloseSoftwareInputPanel();
        }
    }
    
    BorderImage {
        id: background
            source: readOnly?
                platformStyle.backgroundDisabled:
            textEdit.activeFocus? 
                platformStyle.backgroundSelected:
                platformStyle.background

        anchors.fill: parent
        border {
            left: root.platformStyle.backgroundCornerMargin
            right: root.platformStyle.backgroundCornerMargin
            top: root.platformStyle.backgroundCornerMargin
            bottom: root.platformStyle.backgroundCornerMargin
    }*/

    Item {
        id: container
        
        anchors {
            fill: parent
            leftMargin: UI.PADDING_XLARGE; rightMargin: UI.PADDING_XLARGE
            topMargin: UI.PADDING_LARGE; bottomMargin: UI.PADDING_LARGE
        }

        property real horizontalMargins:   container.anchors.leftMargin
                                         + container.anchors.rightMargin
                                         + flick.anchors.leftMargin
                                         + flick.anchors.rightMargin

        property real verticalMargins:  container.anchors.topMargin
                                      + container.anchors.bottomMargin
                                      + flick.anchors.topMargin
                                      + flick.anchors.bottomMargin
        
        clip: true

        Text {
            id: placeholder
            objectName: "placeholder"
            
            property variant model: Text {
                font: textEdit.font
                text: placeholder.text
                visible: false
                wrapMode: textEdit.wrapMode
                horizontalAlignment: textEdit.horizontalAlignment
                verticalAlignment: textEdit.verticalAlignment
                opacity: 0
                
                Binding {
                    when: privy.wrap
                    target: placeholder.model
                    property: "width"
                    value: privy.wrapWidth
                }
            }

            color: root.platformStyle.promptTextColor
            font: textEdit.font
            horizontalAlignment: textEdit.horizontalAlignment
            verticalAlignment: textEdit.verticalAlignment
            visible: {
                if (text && (textEdit.model.paintedWidth == 0 && textEdit.model.paintedHeight <= textEdit.cursorRectangle.height))
                    return (readOnly || !textEdit.activeFocus)
                else
                    return false
            }
            wrapMode: textEdit.wrapMode
            x: textEdit.x; y: textEdit.y
            height: parent.height; width: parent.width
        }

        Flickable {
            id: flick

            property real tiny: Math.round(UI.PADDING_MEDIUM / 2)

            function ensureVisible(rect) {
                if (Math.round(contentX) > Math.round(rect.x))
                    contentX = rect.x
                else if (Math.round(contentX + width) < Math.round(rect.x + rect.width))
                    contentX = rect.x + rect.width - width

                if (Math.round(contentY) > Math.round(rect.y))
                    contentY = rect.y
                else if (Math.round(contentY + height) < Math.round(rect.y + rect.height))
                     contentY = rect.y + rect.height - height
            }

            anchors {
                fill: parent
                leftMargin: tiny
                rightMargin: tiny
                topMargin: tiny / 2
                bottomMargin: tiny / 2
            }

            boundsBehavior: Flickable.StopAtBounds
            contentHeight: textEdit.model.paintedHeight
            contentWidth: textEdit.model.paintedWidth +
                         (textEdit.wrapMode == TextEdit.NoWrap ? textEdit.cursorRectangle.width : 0)
            interactive: root.enabled

            onHeightChanged: {
                if(textEdit.cursorVisible || textEdit.cursorPosition == textEdit.selectionEnd)
                    ensureVisible(textEdit.cursorRectangle)
            }

            onWidthChanged: {
                if(textEdit.cursorVisible || textEdit.cursorPosition == textEdit.selectionEnd)
                    ensureVisible(textEdit.cursorRectangle)
            }

            Component.onCompleted: { scrollDecorator.flickableItem = flick }

            TextEdit {
                id: textEdit
                objectName: "textEdit"

                property variant model: TextEdit {
                    font: textEdit.font
                    text: textEdit.text
                    horizontalAlignment: textEdit.horizontalAlignment
                    verticalAlignment: textEdit.verticalAlignment
                    wrapMode: textEdit.wrapMode
                    visible: false
                    opacity: 0


                    // In Wrap mode, if the width is bound the text will always get wrapped at the
                    // set width. If the width is not bound the text won't get wrapped but
                    // paintedWidth will increase.
                    Binding {
                        when: privy.wrap
                        target: textEdit.model
                        property: "width"
                        value: privy.wrapWidth
                    }
                }
                activeFocusOnPress: true
                enabled: root.enabled
                focus: true
                font: root.platformStyle.textFont
                color: root.platformStyle.textColor
                cursorVisible: activeFocus && !selectedText
                selectedTextColor: root.platformStyle.selectedTextColor
                selectionColor: root.platformStyle.selectionColor

                textFormat: TextEdit.AutoText
                wrapMode: TextEdit.Wrap
                height: flick.height
                width: flick.width
                // TODO: Make bug report?
                // Called too early (before TextEdit size is adjusted) delay ensureVisible call a bit
                onCursorRectangleChanged: delayedEnsureVisible.start()
                /*onActiveFocusChanged: {
                    if (activeFocus) {
                        horizontal.flash()
                        vertical.flash()
                    }
                }*/
                onEnabledChanged: {
                    if (!enabled) {
                        deselect()
                        // De-focusing requires setting focus elsewhere, in this case editor's parent
                        if (root.parent)
                            root.parent.forceActiveFocus()
                    }
                }


                /*Keys.forwardTo: touchController
                
                TextTouchController {
                    id: touchController

                    //  selection handles require touch area geometry to differ from TextEdit's geometry
                    anchors {
                        top: editor.top; topMargin: -container.verticalMargins / 2
                        left: editor.left; leftMargin: -container.horizontalMargins / 2
                    }
                    height: Math.max(root.height, flick.contentHeight + container.verticalMargins * 2)
                    width: Math.max(root.width, flick.contentWidth + container.horizontalMargins * 2)
                    editorScrolledX: flick.contentX - (container.horizontalMargins / 2)
                    editorScrolledY: flick.contentY - (container.verticalMargins / 2)
                    copyEnabled: textEdit.selectedText
                    cutEnabled: !textEdit.readOnly && textEdit.selectedText

                    Component.onCompleted: flick.movementEnded.connect(touchController.flickEnded)
                    Connections { target: screen; onCurrentOrientationChanged: touchController.updateGeometry() }
                    Connections {
                        target: textEdit
                        onHeightChanged: touchController.updateGeometry()
                        onWidthChanged: touchController.updateGeometry()
                        onHorizontalAlignmentChanged: touchController.updateGeometry()
                        onVerticalAlignmentChanged: touchController.updateGeometry()
                        onWrapModeChanged: touchController.updateGeometry()
                        onFontChanged: touchController.updateGeometry()
                    }
                }*/
            }
            
            PropertyAnimation { id: fade; target: textEdit; property: "opacity"; from: 0; to: 1; duration: 250 }
            PropertyAnimation { id: scroll; target: flick; property: "contentY"; duration: 250 }
        }

        ScrollDecorator {
            id: scrollDecorator
        }

        /*ScrollBar {
            id: vertical
            anchors {
                top: flick.top;
                topMargin: -flick.tiny / 2;
                left: flick.right;
            }
            flickableItem: flick
            interactive: false
            orientation: Qt.Vertical
            height: container.height
        }*/

        Timer {
            id: delayedEnsureVisible
            interval: 1
            onTriggered: flick.ensureVisible(textEdit.cursorRectangle)
        }
    }
}
