import QtQuick 1.1
import com.nokia.meego 1.0

Item {
    id: rootZoomableImage
    signal clicked()
    property string source
    property Item photo: staticPicture
    property variant status: photo.status

    state: "STATIC"

    Flickable {
        id: flickable
        clip: true
        width:  parent.width
        height: parent.height
        contentWidth: imageContainer.width
        contentHeight: imageContainer.height
        onHeightChanged: imageContainer.calculateSize()

        Item {
            id: imageContainer
            property real prevScale
            width: Math.max(photo.width * photo.scale, flickable.width)
            height: Math.max(photo.height * photo.scale, flickable.height)

            function calculateSize() {
                var tmpScale = Math.min(flickable.width / photo.width, flickable.height / photo.height) * 0.98;
                photo.scale = Math.min(tmpScale, 1);
                prevScale = photo.scale;
                pinchArea.minScale = photo.scale;
            }

            Image {
                id: staticPicture

                visible: false

                anchors.centerIn: parent

                sourceSize.width: 1024

                smooth: !flickable.movingVertically
                fillMode: Image.PreserveAspectFit

                onStatusChanged: {
                    if(status == Image.Loading) {
                        busyIndicator.opacity = 1;
                    } 
                    else {
                        if(status == Image.Error) {
                            busyIndicator.opacity = 0;
                        }
                        else {
                            if(status == Image.Ready) {
                                if(sourceSize.height > 1024) {
                                    sourceSize.width = undefined;
                                    sourceSize.height = 1024;
                                }

                                busyIndicator.opacity = 0;
                                imageContainer.calculateSize();
                            }
                        }
                    }
                }

                onScaleChanged: {
                    if((width * scale) > flickable.width) {
                        var xoff = (flickable.width / 2 + flickable.contentX) * scale / imageContainer.prevScale;
                        flickable.contentX = xoff - flickable.width / 2;
                    }
                    if((height * scale) > flickable.height) {
                        var yoff = (flickable.height / 2 + flickable.contentY) * scale / imageContainer.prevScale;
                        flickable.contentY = yoff - flickable.height / 2;
                    }

                    imageContainer.prevScale = scale;
                }
            }

            AnimatedImage {
                id: animatedPicture

                visible: false

                anchors.centerIn: parent

                smooth: !flickable.movingVertically
                fillMode: Image.PreserveAspectFit

                onStatusChanged: {
                    if(status == Image.Loading) {
                        busyIndicator.opacity = 1;
                    }
                    else {
                        if(status == Image.Error) {
                            busyIndicator.opacity = 0;
                        }
                        else {
                            if(status == Image.Ready) {
                                busyIndicator.opacity = 0;
                                imageContainer.calculateSize();
                                playing = true;
                            }
                        }
                    }
                }

                onScaleChanged: {
                    if((width * scale) > flickable.width) {
                        var xoff = (flickable.width / 2 + flickable.contentX) * scale / imageContainer.prevScale;
                        flickable.contentX = xoff - flickable.width / 2;
                    }
                    if((height * scale) > flickable.height) {
                        var yoff = (flickable.height / 2 + flickable.contentY) * scale / imageContainer.prevScale;
                        flickable.contentY = yoff - flickable.height / 2;
                    }

                    imageContainer.prevScale = scale;
                }
            }
        }

        PinchArea {
            id: pinchArea
            property real minScale:  1.0
            property real maxScale: 4.0

            anchors.fill: parent

            pinch.target: photo
            pinch.minimumScale: minScale / 2
            pinch.maximumScale: 8.0

            onPinchStarted: if(state === "ANIMATED") photo.playing = false;

            onPinchFinished: {
                if(photo.scale > pinchArea.maxScale) {
                    resetToMaxAnimation.to = pinchArea.maxScale;
                    resetToMaxAnimation.start();
                }
                if(photo.scale < pinchArea.minScale) {
                    resetToMaxAnimation.to = pinchArea.minScale;
                    resetToMaxAnimation.start();
                }

                flickable.returnToBounds();
                if(state === "ANIMATED")
                    photo.playing = true;
            }
        }

        NumberAnimation {
            id: resetToMaxAnimation;
            target: photo
            properties: "scale"
            duration: 100
            onStarted: {
                photo.smooth = !running;
                if(state === "ANIMATED")
                    photo.playing = !running;
            }
        }
        
        MouseArea {
            id: mousearea
            anchors.fill: parent
            property bool doubleClicked:  false

            enabled: photo.status !== Image.Loading

            Timer {
                id: clickTimer
                interval: 520
                onTriggered: rootZoomableImage.clicked()
                running: false
                repeat:  false
            }

            onDoubleClicked: {
                clickTimer.stop();
                mouse.accepted = true;
                if(photo.scale > pinchArea.minScale) {
                    resetToMaxAnimation.to = pinchArea.minScale;
                    resetToMaxAnimation.start();
                    flickable.returnToBounds();
                }
                else {
                    resetToMaxAnimation.to = pinchArea.maxScale;
                    resetToMaxAnimation.start();
                }
            }

            onClicked: clickTimer.start()
        }
    }

    onSourceChanged: {
        if(source.length === 0)
            return;
        if(source.indexOf(".gif") === source.length - 4) {
            state = "ANIMATED";
        }
        else {
            state = "STATIC";
        }
    }

    states: [
        State {
             name: "STATIC"
             PropertyChanges { target: rootZoomableImage; photo: staticPicture; }
             PropertyChanges { target: staticPicture; visible: true; }
             PropertyChanges { target: animatedPicture; visible: false; }
             PropertyChanges { target: staticPicture; source: rootZoomableImage.source; }
             PropertyChanges { target: animatedPicture; source: ""; }
        },
        State {
             name: "ANIMATED"
             PropertyChanges { target: rootZoomableImage; photo: animatedPicture; }
             PropertyChanges { target: staticPicture; visible: false; }
             PropertyChanges { target: animatedPicture; visible: true; }
             PropertyChanges { target: staticPicture; source: ""; }
             PropertyChanges { target: animatedPicture; source: rootZoomableImage.source; }
        }
    ]
}
